//////////////////////////////////////////////////////////////////////////////
//
// $Id: logger.h,v 1.4 2001/07/21 10:27:12 assman Exp $
// a quick implementation for a multi-purpouse logger
// Juan M. Casillas
// assman@gsyc.inf.uc3m.es
// 20/may/2001 (sat 13:25)
//
// $Log: logger.h,v $
// Revision 1.4  2001/07/21 10:27:12  assman
// Added semaphore interface
// Added functions to manage usec resolution time
// fixed bugfixes
// this is the first public release of the library
//
// Revision 1.3  2001/06/24 13:43:12  assman
// added full cvs tags
//
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __LOGGER_H__
#define __LOGGER_H__

#include <string>
#include <buildstr>
#include <errormgmt>
#include <fstream>

#include <sys/utsname.h>
#include <time.h>
#include <strings.h>
#include <unistd.h>
#include <syslog.h>

namespace LOGGER {

  namespace ERROR {
    extern const int success;
    extern const int badlogfile;
    extern const int badformat;
    extern const int nullfmt;

    extern const char *errlist[];
  }

  namespace LEVEL {
    extern const int EMERG;
    extern const int ALERT;
    extern const int CRIT;
    extern const int ERR;
    extern const int WARNING;
    extern const int NOTICE;
    extern const int INFO;
    extern const int DEBUG;

    extern const char *labels[];
  }
}

//////////////////////////////////////////////////////////////////////////////
//
// a class to manage Errors
//
//////////////////////////////////////////////////////////////////////////////

class LoggerError: public GenericError {
public:
  LoggerError()  { clearerr(); }
  int errnum(void) { return(_errnum); }
  string strerr(void) {
      string ret = LOGGER::ERROR::errlist[_errnum]; 
    return(ret); 
  }
  void clearerr(void) {
    _errnum = LOGGER::ERROR::success;
  }

protected:
  int _errnum;
};


//////////////////////////////////////////////////////////////////////////////
//
// The base class for the loggers
//
//////////////////////////////////////////////////////////////////////////////

class BaseLogger {
public:

    BaseLogger():_loglevel(LOGGER::LEVEL::DEBUG) { }

    virtual void log(int level, const char *fmt, ...) = 0;
    int loglevel(int level) {
	int ret = 1;
	if ((level <= LOGGER::LEVEL::EMERG) &&
	    (level >= LOGGER::LEVEL::DEBUG))
	    _loglevel = level; 
	else
	    ret = 0;
	return(ret);
    }
    
 protected:
    virtual void _process_log_entry(int level, string str) = 0;
    int _loglevel;
};


//////////////////////////////////////////////////////////////////////////////
//
// The class that haves the log function and the error clases
// all the loggers should inherit from this class
//
//////////////////////////////////////////////////////////////////////////////


class Logger: public BaseLogger, 
	      public LoggerError {

public:
    Logger(): BaseLogger(), LoggerError() { }
    void log(int level, const char *fmt, ...);
    
protected:
    virtual void _process_log_entry(int level, string str) = 0;
};

 

//////////////////////////////////////////////////////////////////////////////
//
// this class does nothing
//
//////////////////////////////////////////////////////////////////////////////

class EmptyLogger: public Logger {
 public:
    EmptyLogger(): Logger() { }
    virtual ~EmptyLogger() { }
    
    void log(int level, const char *fmt, ...) { }

 protected:
    void _process_log_entry(int level, string str) { }
};

//////////////////////////////////////////////////////////////////////////////
//
// this class output the log from the standard error 
//
//////////////////////////////////////////////////////////////////////////////

class ErrOutputLogger: public Logger {
public:
  ErrOutputLogger(string ident="", int pid=false, int thr_id=false): 
    Logger(),
    _ident(ident),
    _pid(pid),
    _thr_id(thr_id) {
    //
    // get host info
    //
    utsname hinfo;
    uname(&hinfo);
    _hostname = hinfo.nodename;
  }

  virtual ~ErrOutputLogger() {} 

  static const bool logpid;
  static const bool logthreadpid;


protected:

  void _process_log_entry(int level, string str);
  virtual void _display_log(string str);

  string _ident;
  bool    _pid;
  bool    _thr_id;
  string _hostname;

};


//////////////////////////////////////////////////////////////////////////////
//
// a class to log to a file
//
//////////////////////////////////////////////////////////////////////////////

class FileLogger: public ErrOutputLogger {
public:

  FileLogger(string filename, 
	       string ident="", 
	       int pid=false, int thr_id=false):
    ErrOutputLogger(ident, pid,thr_id),
    _filename(filename) { open(); }

  virtual ~FileLogger() { close(); } 

  void open(void) {
    _fstream.open(_filename.c_str(), ios::app);
    if (!_fstream) {
      _errnum = LOGGER::ERROR::badlogfile;
      return;
    }
  }
  void close(void) {
    _fstream.close();
  }



protected:

  void _display_log(string str) {
      _fstream << str << endl;
  }

  string  _filename;
  ofstream _fstream;

};

//////////////////////////////////////////////////////////////////////////////
//
// a class to log to the syslog
//
//////////////////////////////////////////////////////////////////////////////

class SyslogLogger: public Logger {
public:
  SyslogLogger(const char *ident=NULL, 
	       int option=LOG_PID, int facility=LOG_USER): 
    Logger(),
    _ident(ident),
    _option(option),
    _facility(facility) { open(); }

  virtual ~SyslogLogger() { close(); } 

  void open(void) { openlog(_ident,_option,_facility); }   
  void close(void) { closelog(); }

protected:

  void _process_log_entry(int level, string str) {
      syslog(level,"%s",str.c_str());
  }

  const char *_ident;
  int    _option;
  int    _facility;
};


namespace LOGGER {

  //
  // a simple factory to generate logger classes 
  //

  Logger *Factory(string what, string servername="");
}

#endif
